@@ -202,7 +202,18 @@ def add_new_key(key):
 
 def add_key_remote(key):
     try:
-        subprocess.run(["apt-key", "adv", "--keyserver", "hkps://keyserver.ubuntu.com:443", "--recv-keys", key], check=True)
+        proxy = []
+        if os.environ.get('http_proxy') is not None:
+            if os.environ.get('http_proxy') != "":
+                # might be an good idea to check the environment variable content for url only ....
+                # simple way could be
+                # try:
+                #     urllib.urlopen(url)
+                # except IOError:
+                #     print "Not a real URL"
+                # is unclear to me about the fault message
+                proxy=["--keyserver-options", "http-proxy=%s" % os.environ.get('http_proxy')]
+        subprocess.run(["apt-key", "adv", "--keyserver", "hkps://keyserver.ubuntu.com:443"] + proxy + ["--recv-keys", key], check=True)
     except subprocess.CalledProcessError:
         return False
     return True
@@ -436,7 +447,7 @@ class Repository():
                                     name = subparts[-2].capitalize()
                                 else:
                                     name = subparts[-3].capitalize()
-                            name = name.replace("Linuxpearl", "Pearl Linux")
+                            name = name.replace("Pearllinux", "Pearl Linux")
                             name = name.replace("01", "Intel")
                             name = name.replace("Steampowered", "Steam")
                         break
@@ -729,6 +740,7 @@ class MirrorSelectionDialog(object):
         self.local_country_code = cur_country_code or os.environ.get('LANG', 'US').split('.')[0].split('_')[-1]  # fallback to LANG location or 'US'
 
         self.bordering_countries = []
+        self.network_neighbors = []
         self.subregion = []
         self.region = []
         self.local_country = self.get_country(self.local_country_code)
@@ -742,10 +754,13 @@ class MirrorSelectionDialog(object):
                         self.region.append(country_code)
                 if country["cca3"] in self.local_country["borders"]:
                     self.bordering_countries.append(country_code)
+                elif country["cca3"] in self.local_country["networkNeighbors"]:
+                    self.network_neighbors.append(country_code)
 
         self.worldwide_mirrors = []
         self.local_mirrors = []
         self.bordering_mirrors = []
+        self.network_neighbors_mirrors = []
         self.subregional_mirrors = []
         self.regional_mirrors = []
         self.official_mirrors = []
@@ -759,6 +774,8 @@ class MirrorSelectionDialog(object):
                 self.local_mirrors.append(mirror)
             elif mirror.country_code in self.bordering_countries:
                 self.bordering_mirrors.append(mirror)
+            elif mirror.country_code in self.network_neighbors:
+                self.network_neighbors_mirrors.append(mirror)
             elif mirror.country_code in self.subregion:
                 self.subregional_mirrors.append(mirror)
             elif mirror.country_code in self.region:
@@ -770,20 +787,17 @@ class MirrorSelectionDialog(object):
 
         self.worldwide_mirrors = sorted(self.worldwide_mirrors, key=lambda x: x.country_code)
         self.bordering_mirrors = sorted(self.bordering_mirrors, key=lambda x: x.country_code)
+        self.network_neighbors_mirrors = sorted(self.network_neighbors_mirrors, key=lambda x: x.country_code)
         self.subregional_mirrors = sorted(self.subregional_mirrors, key=lambda x: x.country_code)
         self.regional_mirrors = sorted(self.regional_mirrors, key=lambda x: x.country_code)
 
-        self.visible_mirrors = self.worldwide_mirrors + self.local_mirrors + self.bordering_mirrors + self.subregional_mirrors + self.regional_mirrors + self.official_mirrors
-
-        if self.local_country_code in ["IL"]:
-            # For some countries, geographical proximity doesn't equate to faster mirrors.
-            self.visible_mirrors = self.visible_mirrors + self.other_mirrors
+        self.visible_mirrors = self.worldwide_mirrors + self.local_mirrors + self.bordering_mirrors + self.network_neighbors_mirrors + self.subregional_mirrors + self.regional_mirrors + self.official_mirrors
 
         if len(self.visible_mirrors) < 2:
             # We failed to identify the continent/country, let's show all mirrors
             self.visible_mirrors = mirrors
 
-        # Try to find the age of the Pearl archive
+        # Try to find the age of the Mint archive
         self.default_mirror_age = None
         self.default_mirror_date = None
 
@@ -1117,7 +1131,7 @@ class Application(object):
 
         # Now parse other sources and remove any duplicates
         found_duplicates = False
-        for listfile in ["/etc/apt/sources.list"] + glob.glob("/etc/apt/sources.list.d/*.list"):
+        for listfile in glob.glob("/etc/apt/sources.list") + glob.glob("/etc/apt/sources.list.d/*.list"):
             if not listfile.startswith("/etc/apt/sources.list.d/official"):
                 with open(listfile, encoding="utf-8", errors="ignore") as f:
                     lines = []
@@ -1671,17 +1685,17 @@ class Application(object):
 
     def select_new_mirror(self, widget):
         url = self.mirror_selection_dialog.run(self.mirrors, self.config, False)
-        if url is not None:
+        if url is not None and self.selected_mirror != url:
             self.selected_mirror = url
             self.builder.get_object("label_mirror_name").set_text(self.selected_mirror)
-        self.apply_official_sources()
+            self.apply_official_sources()
 
     def select_new_base_mirror(self, widget):
         url = self.mirror_selection_dialog.run(self.base_mirrors, self.config, True)
-        if url is not None:
+        if url is not None and self.selected_base_mirror != url:
             self.selected_base_mirror = url
             self.builder.get_object("label_base_mirror_name").set_text(self.selected_base_mirror)
-        self.apply_official_sources()
+            self.apply_official_sources()
 
     def run(self):
         self._main_window.show_all()
@@ -1896,7 +1910,7 @@ if __name__ == "__main__":
         print ("LSB codename: '%s'." % lsb_codename)
         if os.path.exists("/etc/pearllinux/info"):
             print ("Version of base-files: '%s'." % subprocess.getoutput("dpkg-query -f '${Version}' -W base-files"))
-            print ("Your LSB codename isn't a valid Pearl Linux codename.")
+            print ("Your LSB codename isn't a valid Pearl codename.")
         else:
             print ("This codename isn't currently supported.")
         print ("Please check your LSB information with \"lsb_release -a\".")
